use strict;
use warnings FATAL => 'all';

use Test::More;
use Test::Warnings;

#use Dist::Zilla::Tester;

pass "I can't think of how I can test this without swapping out large amounts of code in doing so";
done_testing;
